//
//  SDK_connect.m
//  SampleCodeOBJC
//
//  Created by Ivan Mao on 12-8-22.
//  Copyright 2012 PQ Labs. All rights reserved.
//

#import "SDK_connect.h"

@implementation SDK_connect

-(id)init
{
	self = [super init];
	
	int err_code = PQMTE_SUCCESS;
	
	if (self != nil) {
		connect_PQ = [[PQMTClient alloc] init];		//register delegate
		connect_PQ.delegate = self;					//register delegate
		
		// initialize the handle functions of gestures;
		[self InitFuncOnTG];
		
		// connect default server ps: ip="127.0.0.1" port=21555
		if((err_code = [PQMTClient ConnectServer:NULL :PQMT_DEFAULT_CLIENT_PORT]) != PQMTE_SUCCESS)
		{
			NSAlert *alert = [[[NSAlert alloc] init] autorelease];
			[alert setMessageText:@"Coonect Server fail!"];
			[alert setInformativeText:[NSString stringWithFormat:@"error code:%d",err_code]];
			[alert runModal];
			NSLog(@"Connect server fail!");
			exit(0);
		}
		
		// send request to server
		TouchClientRequest tcp = {0}; 
		tcp.type = RQST_RAWDATA_ALL | RQST_GESTURE_ALL;
		
		if ((err_code = [PQMTClient SendRequest:tcp]) != PQMTE_SUCCESS) {
			NSAlert *alert = [[[NSAlert alloc] init] autorelease];
			[alert setMessageText:@"send request fail!"];
			[alert setInformativeText:[NSString stringWithFormat:@"error code:%d",err_code]];
			[alert runModal];
			NSLog(@"send request fail!");
			exit(0);
		}
		
		//get server resolution
		[PQMTClient GetServerResolution];
	}
	
	return self;
}

-(void)dealloc
{
	[PQMTClient DisconnectServer];
	[super dealloc];
	[connect_PQ release];
}

-(void)InitFuncOnTG
{
	m_pf_on_tges[TG_TOUCH_START] = @selector(OnTG_TouchStart::);
	m_pf_on_tges[TG_DOWN] = @selector(OnTG_Down::);
	m_pf_on_tges[TG_MOVE] = @selector(OnTG_Move::);
	m_pf_on_tges[TG_UP] = @selector(OnTG_Up::);
	
	m_pf_on_tges[TG_SECOND_DOWN] = @selector(OnTG_SecondDown::);
	m_pf_on_tges[TG_SECOND_UP] = @selector(OnTG_SecondUp::);
	
	m_pf_on_tges[TG_SPLIT_START] = @selector(OnTG_SplitStart::);
	m_pf_on_tges[TG_SPLIT_APART] = @selector(OnTG_SplitApart::);
	m_pf_on_tges[TG_SPLIT_CLOSE] = @selector(OnTG_SplitClose::);
	m_pf_on_tges[TG_SPLIT_END] = @selector(OnTG_SplitEnd::);
	
	m_pf_on_tges[TG_ROTATE_START] = @selector(OnTG_RotateStart::);
	m_pf_on_tges[TG_ROTATE_CLOCKWISE] = @selector(OnTG_RotateClock::);
	m_pf_on_tges[TG_ROTATE_ANTICLOCKWISE] = @selector(OnTG_RotateAnticlock::);
	m_pf_on_tges[TG_ROTATE_END] = @selector(OnTG_RotateEnd::);
	
	m_pf_on_tges[TG_TOUCH_END] = @selector(OnTG_TouchEnd::);
}

-(void)OnReceiveGesture:(const TouchGesture &)ges
{
	if([self respondsToSelector:m_pf_on_tges[ges.type]])
	{
		pf_on_ges = (void (*)(id, SEL, const TouchGesture &, void *)) [self methodForSelector:m_pf_on_tges[ges.type]];
		pf_on_ges(self, m_pf_on_tges[ges.type], ges, NULL);
	}
	
	return;
}

-(void)OnReceivePointFrame:(int)frame_id :(int)time_stamp :(int)point_count :(const TouchPoint *)point_array
{
	int i;
	NSLog(@"frame_id=%d, time_stamp=%d, point_count=%d", frame_id, time_stamp, point_count);
	
	for(i=0; i<point_count; i++)
	{
		TouchPoint tp = point_array[i];
		[self OnTouchPoint:tp];
	}
}

-(void)OnServerBreak:(void *)param
{
	NSLog(@"warning: Server Break!");
	[PQMTClient	DisconnectServer];
}

-(void)OnReceiveError:(int)err_code
{
	switch (err_code) {
		case PQMTE_RCV_INVALIDATE_DATA:
			NSLog(@" error: receive invalidate data.");
			break;
		case PQMTE_SERVER_VERSION_OLD:
			NSLog(@" error: the multi-touch server is old for this client, please update the multi-touch server.");
			break;
		default:
			NSLog(@" socket error, socket error code:");
			break;
	}
}

-(void)OnGetServerResolution:(int)max_x :(int)max_y
{
	NSLog(@"Server Resolution: %d, %d", max_x, max_y);		//awake by [PQMTClient GetServerResolution];
	[pencil getResolution:max_x :max_y];
}

-(void)OnGetDeviceInfo:(const TouchDeviceInfo &)deviceinfo
{
}

-(void)OnTouchPoint:(const TouchPoint &)tp
{
	switch (tp.point_event) {
		case TP_DOWN:
			[pencil addMyTouchPoint:tp.id];
			NSLog(@"Point %d come at (%d,%d) width: %d, height: %d", tp.id, tp.x, tp.y, tp.dx, tp.dy);
			break;
		case TP_MOVE:
			{
				NSPoint tmpPoint = NSMakePoint(tp.x, tp.y);
				[pencil drawMyTouchPoint:tmpPoint: tp.id];
				 NSLog(@"Point %d move at (%d,%d) width: %d, height: %d", tp.id, tp.x, tp.y, tp.dx, tp.dy);
			}
			break;
		case TP_UP:
			[pencil deleteMyTouchPoint:tp.id];
			NSLog(@"Point %d leave at (%d,%d) width: %d, height: %d", tp.id, tp.x, tp.y, tp.dx, tp.dy);
			break;
		default:
			break;
	}
}

-(void)OnTG_TouchStart:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"touch start");
	//[PQMTClient GetServerResolution];
	[Output OnTG_TouchStart_doSomething];
}

-(void)OnTG_Down:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"touch down");
	[Output OnTG_Down_doSomething];
}

-(void)OnTG_Move:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"touch move");
	[Output OnTG_Move_doSomething];
}

-(void)OnTG_Up:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"touch up");
	[Output OnTG_Up_doSomething];
}

-(void)OnTG_TouchEnd:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"touch end");
	[Output OnTG_TouchEnd_doSomething];
}

-(void)OnTG_SecondDown:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"touch second down");
	[Output OnTG_SecondDown_doSomething];
}

-(void)OnTG_SecondUp:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"touch second up");
	[Output OnTG_SecondUp_doSomething];
}

-(void)OnTG_SplitStart:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"split start");
	[Output OnTG_SplitStart_doSomething];
}

-(void)OnTG_SplitApart:(const TouchGesture &)tg :(void *) call_object
{
	NSLog(@"split apart");
	[Output OnTG_SplitApart_doSomething];
}

-(void)OnTG_SplitClose:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"split close");
	[Output OnTG_SplitClose_doSomething];
}

-(void)OnTG_SplitEnd:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"split end");
	[Output OnTG_SplitEnd_doSomething];
}

-(void)OnTG_RotateStart:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"rotate start");
	[Output OnTG_RotateStart_doSomething];
}

-(void)OnTG_RotateClock:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"rotate clock");
	[Output OnTG_RotateClock_doSomething];
}

-(void)OnTG_RotateAnticlock:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"rotate anticlock");
	[Output OnTG_RotateAnticlock_doSomething];
}

-(void)OnTG_RotateEnd:(const TouchGesture &)tg :(void *)call_object
{
	NSLog(@"rotate end");
	[Output OnTG_RotateEnd_doSomething];
}


@end
